// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       message.h
 *  \brief      表示メッセージ
 *  \date       Since: February 8, 2023. 22:28:08 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MESSAGE_H_1675862888
#define INCGUARD_MESSAGE_H_1675862888

#include <mtk/text_format.h>

#include "message/message_data.h"

namespace msgconv
{
void InitMessage() noexcept;
void PutMessage(MsgID msgID, const MTK::FormatArgs &args = {}) noexcept;
void PutError(MsgID msgID, const MTK::FormatArgs &args = {}) noexcept;
void PutWarning(MsgID msgID, const MTK::FormatArgs &args = {}) noexcept;

/* ------------------------------------------------------------------------- */
/*!
 *  \brief      メッセージの表示
 *  \param[in]  msgID   メッセージID
 *  \param[in]  args    フォーマット引数
 */
/* ------------------------------------------------------------------------- */
template <class... Args>
void PutMessage(MsgID msgID, const Args&... args) noexcept
{
    return PutMessage(msgID, {args...});
}

/* ------------------------------------------------------------------------- */
/*!
 *  \brief      エラーメッセージの表示
 *  \param[in]  msgID   メッセージID
 *  \param[in]  args    フォーマット引数
 */
/* ------------------------------------------------------------------------- */
template <class... Args>
void PutError(MsgID msgID, const Args&... args) noexcept
{
    return PutError(msgID, {args...});
}

/* ------------------------------------------------------------------------- */
/*!
 *  \brief      警告メッセージの表示
 *  \param[in]  msgID   メッセージID
 *  \param[in]  args    フォーマット引数
 */
/* ------------------------------------------------------------------------- */
template <class... Args>
void PutWarning(MsgID msgID, const Args&... args) noexcept
{
    return PutWarning(msgID, {args...});
}
}   // namespace msgconv

#endif	// INCGUARD_MESSAGE_H_1675862888

// vim: et ts=4 sw=4 sts=4
