// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node.h
 *  \brief      メッセージノード
 *  \date       Since: May 7, 2023. 21:29:23 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_H_1683462563
#define INCGUARD_MSG_NODE_H_1683462563

#include <string>

namespace msgconv
{
//! メッセージノードの種類
enum class MsgNodeType
{
    Text,   //!< テキスト
    Subst,  //!< 置換
    Color,  //!< 色指定
    Face,   //!< 書体指定
    Ruby,   //!< ルビ
    Reset,  //!< タグのリセット
};


//! メッセージノード基底クラス
class MsgNode
{
public:
    virtual ~MsgNode() = default;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept = 0;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept
    {
        return "";
    }
};


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ノードのキャスト
 *  \tparam     T       キャスト先のノードの型
 *  \param[in]  node    キャストするノード
 *  \return     キャスト可能であればその型のノード．不可能であればnullptr
 */
/* ------------------------------------------------------------------------- */
template <class T>
constexpr const T *NodeCast(const MsgNode *node) noexcept
{
    if (node != nullptr)
    {
        if (node->GetNodeType() == T::kNodeType)
        {
            return reinterpret_cast<const T*>(node);
        }
    }

    return nullptr;
}

}   // namespace msgconv
#endif	// INCGUARD_MSG_NODE_H_1683462563

// vim: et ts=4 sw=4 sts=4
