// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node_color.h
 *  \brief      メッセージノード - 色変更タグ
 *  \date       Since: May 8, 2023. 3:04:33 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_COLOR_H_1683482673
#define INCGUARD_MSG_NODE_COLOR_H_1683482673

#include "msg_node.h"

namespace msgconv
{
class ColorMsgNode : public MsgNode
{
public:
    //! ノードタイプの定義
    static constexpr auto kNodeType = MsgNodeType::Color;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  colorName   カラーネーム
     */
    /* ------------------------------------------------------------------------- */
    ColorMsgNode(const std::string &colorName) noexcept
        : _colorName(colorName)
    {
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept override
    {
        return kNodeType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      テキストの取得
     *  \return     テキスト
     */
    /* ------------------------------------------------------------------------- */
    const std::string &GetColorName() const noexcept
    {
        return _colorName;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept override
    {
        return std::string("Color: \"") + _colorName + "\"";
    }

private:
    std::string _colorName;
};
}   // namespace msgconv

#endif	// INCGUARD_MSG_NODE_COLOR_H_1683482673

// vim: et ts=4 sw=4 sts=4
