// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node_face.h
 *  \brief      メッセージノード - 書体
 *  \date       Since: May 9, 2023. 0:55:29 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_FACE_H_1683561329
#define INCGUARD_MSG_NODE_FACE_H_1683561329

#include <string>

#include "msg_node.h"

namespace msgconv
{
class FaceMsgNode : public MsgNode
{
public:
    //! ノードタイプの定義
    static constexpr auto kNodeType = MsgNodeType::Face;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  faceName    書体名
     */
    /* ------------------------------------------------------------------------- */
    FaceMsgNode(const std::string &faceName) noexcept
        : _faceName(faceName)
    {
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept override
    {
        return kNodeType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      書体名の取得
     *  \return     書体名
     */
    /* ------------------------------------------------------------------------- */
    const std::string &GetFaceName() const noexcept
    {
        return _faceName;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept override
    {
        return std::string("Face: \"") + _faceName + "\"";
    }

private:
    std::string _faceName;
};

}   // namespace msgconv

#endif	// INCGUARD_MSG_NODE_FACE_H_1683561329

// vim: et ts=4 sw=4 sts=4
