// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node_reset.h
 *  \brief      メッセージノード - タグのリセット
 *  \date       Since: May 9, 2023. 1:11:41 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_RESET_H_1683562301
#define INCGUARD_MSG_NODE_RESET_H_1683562301

#include <string>

#include <mtk/text_format.h>

#include "msg_node.h"

namespace msgconv
{
//! タグのリセットタイプ
enum class TagResetType
{
    Unknown,    //!< 不明
    Color,      //!< 色
    Face,       //!< 書体
    All         //!< 全部
};

class ResetMsgNode : public MsgNode
{
public:
    //! ノードタイプの定義
    static constexpr auto kNodeType = MsgNodeType::Reset;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  faceName    書体名
     */
    /* ------------------------------------------------------------------------- */
    ResetMsgNode(TagResetType resetType, char letter) noexcept
        : _resetType(resetType)
        , _letter(letter)
    {
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept override
    {
        return kNodeType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      書体名の取得
     *  \return     書体名
     */
    /* ------------------------------------------------------------------------- */
    const TagResetType &GetResetType() const noexcept
    {
        return _resetType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      タグ文字を取得
     */
    /* ------------------------------------------------------------------------- */
    char GetLetter() const noexcept
    {
        return _letter;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept override
    {
        return std::string("Reset: \"") + _letter + "\"";
    }

private:
    TagResetType _resetType;
    char _letter;
};

}   // namespace msgconv
#endif	// INCGUARD_MSG_NODE_RESET_H_1683562301

// vim: et ts=4 sw=4 sts=4
