// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node_subst.h
 *  \brief      メッセージノード - 置換
 *  \date       Since: May 7, 2023. 22:14:09 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_SUBST_H_1683465249
#define INCGUARD_MSG_NODE_SUBST_H_1683465249

#include <string>

#include <mtk/text_format.h>

#include "msg_node.h"

namespace msgconv
{
class SubstMsgNode : public MsgNode
{
public:
    //! ノードタイプの定義
    static constexpr auto kNodeType = MsgNodeType::Subst;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    SubstMsgNode(const MTK::FormatArgument::Option &option) noexcept
        : _option(option)
    {
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept override
    {
        return kNodeType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      置換オプションを取得
     *  \return     置換オプション
     */
    /* ------------------------------------------------------------------------- */
    const MTK::FormatArgument::Option &GetOption() const noexcept
    {
        return _option;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept override
    {
        return std::string("Subst");
    }

private:
    MTK::FormatArgument::Option _option;
};

}   // namespace msgconv

#endif	// INCGUARD_MSG_NODE_SUBST_H_1683465249

// vim: et ts=4 sw=4 sts=4
