// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msg_node_text.h
 *  \brief      メッセージノード - テキスト
 *  \date       Since: May 7, 2023. 21:46:30 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MSG_NODE_TEXT_H_1683463590
#define INCGUARD_MSG_NODE_TEXT_H_1683463590

#include <string>

#include "msg_node.h"

namespace msgconv
{
class TextMsgNode : public MsgNode
{
public:
    //! ノードタイプの定義
    static constexpr auto kNodeType = MsgNodeType::Text;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  text    テキスト
     */
    /* ------------------------------------------------------------------------- */
    TextMsgNode(const std::string &text) noexcept
        : _text(text)
    {
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ノードタイプの取得
     *  \return     ノードタイプ
     */
    /* ------------------------------------------------------------------------- */
    virtual MsgNodeType GetNodeType() const noexcept override
    {
        return kNodeType;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      テキストの取得
     *  \return     テキスト
     */
    /* ------------------------------------------------------------------------- */
    const std::string &GetText() const noexcept
    {
        return _text;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デバッグ用のテキストを取得
     *  \return     デバッグ用のテキスト
     */
    /* ------------------------------------------------------------------------- */
    virtual std::string GetDebugText() const noexcept override
    {
        return std::string("Text: \"") + _text + "\"";
    }

private:
    std::string _text;
};

}   // namespace msgconv

#endif	// INCGUARD_MSG_NODE_TEXT_H_1683463590

// vim: et ts=4 sw=4 sts=4
