// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       text_hash.h
 *  \brief      テキストハッシュ関数定義
 *  \date       Since: October 1, 2019. 16:56:45 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_TEXT_HASH_H_1569916605
#define INCGUARD_TEXT_HASH_H_1569916605

#include <cstdint>

namespace MTK::Hash
{
constexpr uint32_t kFNV1aDefaultValue32  = 0xA3F6C23E;          //!< 32ビット用のデフォルトのハッシュ値
constexpr uint32_t kFNV1aPrime32         = 0x1000193;           //!< 32ビット用の素数
constexpr uint64_t kFNV1aDefaultValue64  = 0xC18F36FE9A341BC5;  //!< 64ビット用のデフォルトのハッシュ値
constexpr uint64_t kFNV1aPrime64         = 0x100000001B3;       //!< 64ビット用の素数


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      FNV-1aアルゴリズムで文字列から32bitハッシュ値を取得
 *  \param[in]  str     文字列
 *  \param[in]  value   デフォルトの32bit値
 *  \return     32bitハッシュ値
 */
/* ------------------------------------------------------------------------- */
constexpr uint32_t FNV1a(const char *str, const uint32_t value = Hash::kFNV1aDefaultValue32) noexcept
{
    if (str[0] == '\0')
    {
        return value;
    }
    
    return FNV1a(&str[1], (value ^ uint32_t(str[0])) * Hash::kFNV1aPrime32);
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      FNV-1aアルゴリズムで文字列から64bitハッシュ値を取得
 *  \param[in]  str     文字列
 *  \param[in]  value   デフォルトの64bit値
 *  \return     64bitハッシュ値
 */
/* ------------------------------------------------------------------------- */
constexpr uint64_t FNV1a64(const char *str, const uint64_t value = Hash::kFNV1aDefaultValue64) noexcept
{
    if (str[0] == '\0')
    {
        return value;
    }
    
    return FNV1a64(&str[1], (value ^ uint64_t(str[0])) * Hash::kFNV1aPrime64);
}
}   // namespace MTK::Hash

#endif  // INCGUARD_TEXT_HASH_H_1569916605

// vim: et ts=4 sw=4 sts=4
