// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       utility.cc
 *  \brief      ユーティリティ関数
 *  \date       Since: October 2, 2019. 10:27:13 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <memory>

#include <mtk/utility.h>

#if defined(MTK_TARGET_WINDOWS)
#include <Windows.h>
#endif

namespace MTK::Utility
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief          文字列の置換
 *  \param[in,out]  str             書き換える文字列
 *  \param[in]      target          変換前の文字列
 *  \param[in]      replacement     変換後の文字列
 *  \return         置き換えた数
 *  \note
 *      引数の説明がややこしいので分かりやすく書くと，
 *      cat str | sed s/target/replacement/g
 */
/* ------------------------------------------------------------------------- */
int ReplaceString(std::string &str, const std::string &target, const std::string &replacement) noexcept
{
    int count = 0;

    std::string::size_type pos = 0;
    while ((pos = str.find(target, pos)) != std::string::npos)
    {
        str.replace(pos, target.length(), replacement);
        pos += replacement.length();
        count++;
    }

    return count;
}

#if defined(MTK_TARGET_WINDOWS)
/* ------------------------------------------------------------------------- */
/*!
 *  \brief          文字列をWindows用のワイド文字に変換
 *  \param[in]      utf8String  変換元の文字列 (UTF-8)
 *  \return         変換結果．失敗時には空の文字列
 */
/* ------------------------------------------------------------------------- */
std::wstring ToWindowsWideChar(const std::string &utf8String) noexcept
{
    auto srcSize = strlen(utf8String.c_str()) + 1;
    size_t bufferSize = srcSize * 3;
    auto buffer = std::make_unique<wchar_t []>(bufferSize);

    MultiByteToWideChar(CP_UTF8, 0, utf8String.c_str(), static_cast<int>(srcSize), buffer.get(), static_cast<int>(bufferSize));

    return std::wstring(buffer.get());
}
#endif


}   // namespace MTK::Utility


// vim: et ts=4 sw=4 sts=4
